package fr.uha.gabalier.core;

import fr.uha.gabalier.util.ImageLib;
import fr.uha.gabalier.view.Preview;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;

public class ImageLoader extends SwingWorker<List<Image>, Integer> {

    private final File root;
    private Preview app;
    private List<Image> images;
    private final int iconWidth;

    public ImageLoader(Preview app, int iconWidth, File root) {
        this.root = root;
        this.app = app;
        this.iconWidth = iconWidth;
        this.images = new ArrayList<>();
    }

    private void loadImages(File current) throws NullPointerException {
        if (current.isFile()) {
            final Image icon = ImageLib.createScaledImage(current, this.iconWidth, -1);
            if (icon != null) {
                images.add(icon);
                publish(images.size());
            }
            return;
        }

        final File[] files = current.listFiles();
        if (files == null) throw new NullPointerException();
        for (File f : files) {
            loadImages(f);
        }
    }

    @Override
    protected List<Image> doInBackground() throws Exception {
        loadImages(root);
        return images;
    }

    @Override
    protected void process(List<Integer> data) {
        if (this.isCancelled()) return;
        app.setStatus(String.valueOf(data.get(data.size() - 1)) + " images loaded");
    }

    @Override
    protected void done() {
        try {
            app.setModel(this.get());
            app.setStatus(app.getModel().size() + " images loaded");
        } catch (CancellationException e) {
            app.setStatus("Cancelled");
        } catch (InterruptedException | ExecutionException e) {
            e.printStackTrace();
        } finally {
            app.postLoad();
        }
    }
}
