package fr.uha.gabalier.view;

import fr.uha.gabalier.controller.ImageController;
import fr.uha.gabalier.core.ImageLoader;

import javax.swing.*;
import java.awt.*;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;
import fr.uha.gabalier.util.languages.StringSources;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;

public class Preview extends JFrame {

	private static final long serialVersionUID = -8104236322964434859L;
	
	static final int VIEW_WIDTH = 256;
    static final int VIEW_HEIGHT = 128;

    public static final String LEFT_COMMAND = "<";
    public static final String RIGHT_COMMAND = ">";
    private static final int ICON_WIDTH = 256;

    private List<Image> model;
    private ImageLoader loader;
    private ImageView imageView;
    protected ImageController<?> controller;

    // UI elements
    private JMenuItem loadItem;
    private JMenuItem cancelItem;
    private JButton rightNavButton;
    private JButton leftNavButton;
    private final JTextField statusText;

	

    public Preview() throws HeadlessException {
    	setIconImage(Toolkit.getDefaultToolkit().getImage(Preview.class.getResource("/fr/uha/gabalier/util/ressources/logo.jpg")));
    	setTitle(StringSources.getString("Preview.this.title")); //$NON-NLS-1$
        model = new ArrayList<>();
        
        this.controller = new ImageController(model);
        
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // UI elements
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);
        JMenu fileMenu = new JMenu(StringSources.getString("Preview.fileMenu.text")); //$NON-NLS-1$
        menuBar.add(fileMenu);

        loadItem = new JMenuItem(StringSources.getString("Preview.loadItem.text")); //$NON-NLS-1$
        fileMenu.add(loadItem);
        loadItem.addActionListener(actionEvent -> doLoad());
        cancelItem = new JMenuItem(StringSources.getString("Preview.cancelItem.text")); //$NON-NLS-1$
        fileMenu.add(cancelItem);
        cancelItem.addActionListener(actionEvent -> doCancel());
        final JMenuItem exitItem = new JMenuItem(StringSources.getString("Preview.exitItem.text")); //$NON-NLS-1$
        exitItem.addActionListener(actionEvent -> doExit());
        exitItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK));
        fileMenu.add(exitItem);

        getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));

        imageView = new ImageView(model);
        imageView.addMouseListener(new MouseAdapter() {
        	@Override
        	public void mouseClicked(MouseEvent e) {
        		JFrame bigViewFrame = new JFrame();
        		bigViewFrame.add(new BigView(model.get(controller.getIndex())));
        		bigViewFrame.repaint();
        		bigViewFrame.setVisible(true);
        	}
        });
        imageView.setController(controller);
        controller.setView(imageView);
        getContentPane().add(imageView);

        final JPanel navPanel = new JPanel();
        getContentPane().add(navPanel);
        leftNavButton = new JButton(LEFT_COMMAND);
        leftNavButton.addActionListener(controller);
        navPanel.add(leftNavButton);
        rightNavButton = new JButton(RIGHT_COMMAND);
        rightNavButton.addActionListener(controller);
        navPanel.add(rightNavButton);

        final JPanel statusPanel = new JPanel();
        getContentPane().add(statusPanel);
        statusText = new JTextField(StringSources.getString("Preview.statusText.text")); //$NON-NLS-1$
        statusPanel.add(new JLabel(StringSources.getString("Preview.other.text"))); //$NON-NLS-1$
        statusPanel.add(statusText);

        pack();
        setVisible(true);
    }

    public void setModel(List<Image> images) {
        this.model.clear();
        this.model.addAll(images);
        ((ImageController) imageView.getController()).resetIndex();
    }

    public List<Image> getModel() {
        return model;
    }

    public void setStatus(String status) {
        this.statusText.setText(status);
    }

    private void doLoad() {
        final JFileChooser fc = new JFileChooser();
        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        final int ret = fc.showOpenDialog(this);
        if (ret == JFileChooser.APPROVE_OPTION) {
            loader = new ImageLoader(this, ICON_WIDTH, fc.getSelectedFile());
            preLoad();
            loader.execute();
        }
    }

    private void doExit() {
        System.exit(0);
    }

    private void doCancel() {
        loader.cancel(true);
    }

    public void preLoad() {
        loadItem.setEnabled(false);
        cancelItem.setEnabled(true);
        rightNavButton.setEnabled(false);
        leftNavButton.setEnabled(false);
    }

    public void postLoad() {
        loadItem.setEnabled(true);
        cancelItem.setEnabled(false);
        rightNavButton.setEnabled(true);
        leftNavButton.setEnabled(true);
    }

    public static void main(String[] args) {
        EventQueue.invokeLater(Preview::new);
    }

}
